"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DType:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DTypeEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DType.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FP8: _DType.ValueType  # 0
    FP16: _DType.ValueType  # 1
    FP32: _DType.ValueType  # 2
    FP64: _DType.ValueType  # 3
    INT8: _DType.ValueType  # 4
    INT16: _DType.ValueType  # 5
    INT32: _DType.ValueType  # 6
    INT64: _DType.ValueType  # 7
    UINT8: _DType.ValueType  # 8
    UINT16: _DType.ValueType  # 9
    UINT32: _DType.ValueType  # 10
    UINT64: _DType.ValueType  # 11

class DType(_DType, metaclass=_DTypeEnumTypeWrapper):
    """----- *
    Types *
    -----
    """

FP8: DType.ValueType  # 0
FP16: DType.ValueType  # 1
FP32: DType.ValueType  # 2
FP64: DType.ValueType  # 3
INT8: DType.ValueType  # 4
INT16: DType.ValueType  # 5
INT32: DType.ValueType  # 6
INT64: DType.ValueType  # 7
UINT8: DType.ValueType  # 8
UINT16: DType.ValueType  # 9
UINT32: DType.ValueType  # 10
UINT64: DType.ValueType  # 11
global___DType = DType

class _JointPositionUnit:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _JointPositionUnitEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_JointPositionUnit.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DEGREES: _JointPositionUnit.ValueType  # 0
    RADIANS: _JointPositionUnit.ValueType  # 1

class JointPositionUnit(_JointPositionUnit, metaclass=_JointPositionUnitEnumTypeWrapper):
    """--------------- *
    Joint Positions *
    ---------------
    """

DEGREES: JointPositionUnit.ValueType  # 0
RADIANS: JointPositionUnit.ValueType  # 1
global___JointPositionUnit = JointPositionUnit

class _JointVelocityUnit:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _JointVelocityUnitEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_JointVelocityUnit.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DEGREES_PER_SECOND: _JointVelocityUnit.ValueType  # 0
    RADIANS_PER_SECOND: _JointVelocityUnit.ValueType  # 1

class JointVelocityUnit(_JointVelocityUnit, metaclass=_JointVelocityUnitEnumTypeWrapper):
    """---------------- *
    Joint Velocities *
    ----------------
    """

DEGREES_PER_SECOND: JointVelocityUnit.ValueType  # 0
RADIANS_PER_SECOND: JointVelocityUnit.ValueType  # 1
global___JointVelocityUnit = JointVelocityUnit

class _JointTorqueUnit:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _JointTorqueUnitEnumTypeWrapper(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_JointTorqueUnit.ValueType], builtins.type
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NEWTON_METERS: _JointTorqueUnit.ValueType  # 0

class JointTorqueUnit(_JointTorqueUnit, metaclass=_JointTorqueUnitEnumTypeWrapper):
    """-------------- *
    Joint Torques *
    --------------
    """

NEWTON_METERS: JointTorqueUnit.ValueType  # 0
global___JointTorqueUnit = JointTorqueUnit

@typing.final
class JointPositionValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOINT_NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    UNIT_FIELD_NUMBER: builtins.int
    joint_name: builtins.str
    value: builtins.float
    unit: global___JointPositionUnit.ValueType
    def __init__(
        self,
        *,
        joint_name: builtins.str = ...,
        value: builtins.float = ...,
        unit: global___JointPositionUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["joint_name", b"joint_name", "unit", b"unit", "value", b"value"]
    ) -> None: ...

global___JointPositionValue = JointPositionValue

@typing.final
class JointPositionsSchema(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_FIELD_NUMBER: builtins.int
    JOINT_NAMES_FIELD_NUMBER: builtins.int
    unit: global___JointPositionUnit.ValueType
    @property
    def joint_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        unit: global___JointPositionUnit.ValueType = ...,
        joint_names: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["joint_names", b"joint_names", "unit", b"unit"]) -> None: ...

global___JointPositionsSchema = JointPositionsSchema

@typing.final
class JointPositionsValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___JointPositionValue]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___JointPositionValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___JointPositionsValue = JointPositionsValue

@typing.final
class JointVelocityValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOINT_NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    UNIT_FIELD_NUMBER: builtins.int
    joint_name: builtins.str
    value: builtins.float
    unit: global___JointVelocityUnit.ValueType
    def __init__(
        self,
        *,
        joint_name: builtins.str = ...,
        value: builtins.float = ...,
        unit: global___JointVelocityUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["joint_name", b"joint_name", "unit", b"unit", "value", b"value"]
    ) -> None: ...

global___JointVelocityValue = JointVelocityValue

@typing.final
class JointVelocitiesSchema(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_FIELD_NUMBER: builtins.int
    JOINT_NAMES_FIELD_NUMBER: builtins.int
    unit: global___JointVelocityUnit.ValueType
    @property
    def joint_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        unit: global___JointVelocityUnit.ValueType = ...,
        joint_names: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["joint_names", b"joint_names", "unit", b"unit"]) -> None: ...

global___JointVelocitiesSchema = JointVelocitiesSchema

@typing.final
class JointVelocitiesValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___JointVelocityValue]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___JointVelocityValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___JointVelocitiesValue = JointVelocitiesValue

@typing.final
class JointTorqueValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOINT_NAME_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    UNIT_FIELD_NUMBER: builtins.int
    joint_name: builtins.str
    value: builtins.float
    unit: global___JointTorqueUnit.ValueType
    def __init__(
        self,
        *,
        joint_name: builtins.str = ...,
        value: builtins.float = ...,
        unit: global___JointTorqueUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["joint_name", b"joint_name", "unit", b"unit", "value", b"value"]
    ) -> None: ...

global___JointTorqueValue = JointTorqueValue

@typing.final
class JointTorquesSchema(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNIT_FIELD_NUMBER: builtins.int
    JOINT_NAMES_FIELD_NUMBER: builtins.int
    unit: global___JointTorqueUnit.ValueType
    @property
    def joint_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        unit: global___JointTorqueUnit.ValueType = ...,
        joint_names: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["joint_names", b"joint_names", "unit", b"unit"]) -> None: ...

global___JointTorquesSchema = JointTorquesSchema

@typing.final
class JointTorquesValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___JointTorqueValue]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___JointTorqueValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___JointTorquesValue = JointTorquesValue

@typing.final
class JointCommandsSchema(google.protobuf.message.Message):
    """---------------- *
    Actuator Command *
    ----------------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOINT_NAMES_FIELD_NUMBER: builtins.int
    TORQUE_UNIT_FIELD_NUMBER: builtins.int
    VELOCITY_UNIT_FIELD_NUMBER: builtins.int
    POSITION_UNIT_FIELD_NUMBER: builtins.int
    torque_unit: global___JointTorqueUnit.ValueType
    velocity_unit: global___JointVelocityUnit.ValueType
    position_unit: global___JointPositionUnit.ValueType
    @property
    def joint_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        joint_names: collections.abc.Iterable[builtins.str] | None = ...,
        torque_unit: global___JointTorqueUnit.ValueType = ...,
        velocity_unit: global___JointVelocityUnit.ValueType = ...,
        position_unit: global___JointPositionUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "joint_names",
            b"joint_names",
            "position_unit",
            b"position_unit",
            "torque_unit",
            b"torque_unit",
            "velocity_unit",
            b"velocity_unit",
        ],
    ) -> None: ...

global___JointCommandsSchema = JointCommandsSchema

@typing.final
class JointCommandValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOINT_NAME_FIELD_NUMBER: builtins.int
    TORQUE_FIELD_NUMBER: builtins.int
    VELOCITY_FIELD_NUMBER: builtins.int
    POSITION_FIELD_NUMBER: builtins.int
    KP_FIELD_NUMBER: builtins.int
    KD_FIELD_NUMBER: builtins.int
    TORQUE_UNIT_FIELD_NUMBER: builtins.int
    VELOCITY_UNIT_FIELD_NUMBER: builtins.int
    POSITION_UNIT_FIELD_NUMBER: builtins.int
    joint_name: builtins.str
    torque: builtins.float
    velocity: builtins.float
    position: builtins.float
    kp: builtins.float
    kd: builtins.float
    torque_unit: global___JointTorqueUnit.ValueType
    velocity_unit: global___JointVelocityUnit.ValueType
    position_unit: global___JointPositionUnit.ValueType
    def __init__(
        self,
        *,
        joint_name: builtins.str = ...,
        torque: builtins.float = ...,
        velocity: builtins.float = ...,
        position: builtins.float = ...,
        kp: builtins.float = ...,
        kd: builtins.float = ...,
        torque_unit: global___JointTorqueUnit.ValueType = ...,
        velocity_unit: global___JointVelocityUnit.ValueType = ...,
        position_unit: global___JointPositionUnit.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "joint_name",
            b"joint_name",
            "kd",
            b"kd",
            "kp",
            b"kp",
            "position",
            b"position",
            "position_unit",
            b"position_unit",
            "torque",
            b"torque",
            "torque_unit",
            b"torque_unit",
            "velocity",
            b"velocity",
            "velocity_unit",
            b"velocity_unit",
        ],
    ) -> None: ...

global___JointCommandValue = JointCommandValue

@typing.final
class JointCommandsValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___JointCommandValue]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___JointCommandValue] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___JointCommandsValue = JointCommandsValue

@typing.final
class CameraFrameSchema(google.protobuf.message.Message):
    """------------ *
    Camera Frame *
    ------------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WIDTH_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    CHANNELS_FIELD_NUMBER: builtins.int
    width: builtins.int
    height: builtins.int
    channels: builtins.int
    def __init__(
        self,
        *,
        width: builtins.int = ...,
        height: builtins.int = ...,
        channels: builtins.int = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["channels", b"channels", "height", b"height", "width", b"width"]
    ) -> None: ...

global___CameraFrameSchema = CameraFrameSchema

@typing.final
class CameraFrameValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___CameraFrameValue = CameraFrameValue

@typing.final
class AudioFrameSchema(google.protobuf.message.Message):
    """----------- *
    Audio Frame *
    -----------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHANNELS_FIELD_NUMBER: builtins.int
    SAMPLE_RATE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    channels: builtins.int
    sample_rate: builtins.int
    dtype: global___DType.ValueType
    def __init__(
        self,
        *,
        channels: builtins.int = ...,
        sample_rate: builtins.int = ...,
        dtype: global___DType.ValueType = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["channels", b"channels", "dtype", b"dtype", "sample_rate", b"sample_rate"]
    ) -> None: ...

global___AudioFrameSchema = AudioFrameSchema

@typing.final
class AudioFrameValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___AudioFrameValue = AudioFrameValue

@typing.final
class ImuAccelerometerValue(google.protobuf.message.Message):
    """--- *
    IMU *
    ---
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___ImuAccelerometerValue = ImuAccelerometerValue

@typing.final
class ImuGyroscopeValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___ImuGyroscopeValue = ImuGyroscopeValue

@typing.final
class ImuMagnetometerValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X_FIELD_NUMBER: builtins.int
    Y_FIELD_NUMBER: builtins.int
    Z_FIELD_NUMBER: builtins.int
    x: builtins.float
    y: builtins.float
    z: builtins.float
    def __init__(
        self,
        *,
        x: builtins.float = ...,
        y: builtins.float = ...,
        z: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["x", b"x", "y", b"y", "z", b"z"]) -> None: ...

global___ImuMagnetometerValue = ImuMagnetometerValue

@typing.final
class ImuSchema(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    USE_ACCELEROMETER_FIELD_NUMBER: builtins.int
    USE_GYROSCOPE_FIELD_NUMBER: builtins.int
    USE_MAGNETOMETER_FIELD_NUMBER: builtins.int
    use_accelerometer: builtins.bool
    use_gyroscope: builtins.bool
    use_magnetometer: builtins.bool
    def __init__(
        self,
        *,
        use_accelerometer: builtins.bool = ...,
        use_gyroscope: builtins.bool = ...,
        use_magnetometer: builtins.bool = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "use_accelerometer",
            b"use_accelerometer",
            "use_gyroscope",
            b"use_gyroscope",
            "use_magnetometer",
            b"use_magnetometer",
        ],
    ) -> None: ...

global___ImuSchema = ImuSchema

@typing.final
class ImuValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LINEAR_ACCELERATION_FIELD_NUMBER: builtins.int
    ANGULAR_VELOCITY_FIELD_NUMBER: builtins.int
    MAGNETIC_FIELD_FIELD_NUMBER: builtins.int
    @property
    def linear_acceleration(self) -> global___ImuAccelerometerValue: ...
    @property
    def angular_velocity(self) -> global___ImuGyroscopeValue: ...
    @property
    def magnetic_field(self) -> global___ImuMagnetometerValue: ...
    def __init__(
        self,
        *,
        linear_acceleration: global___ImuAccelerometerValue | None = ...,
        angular_velocity: global___ImuGyroscopeValue | None = ...,
        magnetic_field: global___ImuMagnetometerValue | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "angular_velocity",
            b"angular_velocity",
            "linear_acceleration",
            b"linear_acceleration",
            "magnetic_field",
            b"magnetic_field",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "angular_velocity",
            b"angular_velocity",
            "linear_acceleration",
            b"linear_acceleration",
            "magnetic_field",
            b"magnetic_field",
        ],
    ) -> None: ...

global___ImuValue = ImuValue

@typing.final
class TimestampSchema(google.protobuf.message.Message):
    """--------- *
    Timestamp *
    ---------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    START_SECONDS_FIELD_NUMBER: builtins.int
    START_NANOS_FIELD_NUMBER: builtins.int
    start_seconds: builtins.int
    start_nanos: builtins.int
    def __init__(
        self,
        *,
        start_seconds: builtins.int = ...,
        start_nanos: builtins.int = ...,
    ) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["start_nanos", b"start_nanos", "start_seconds", b"start_seconds"]
    ) -> None: ...

global___TimestampSchema = TimestampSchema

@typing.final
class TimestampValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECONDS_FIELD_NUMBER: builtins.int
    NANOS_FIELD_NUMBER: builtins.int
    seconds: builtins.int
    nanos: builtins.int
    def __init__(
        self,
        *,
        seconds: builtins.int = ...,
        nanos: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["nanos", b"nanos", "seconds", b"seconds"]) -> None: ...

global___TimestampValue = TimestampValue

@typing.final
class VectorCommandSchema(google.protobuf.message.Message):
    """------------------- *
    XY Position Command *
    -------------------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DIMENSIONS_FIELD_NUMBER: builtins.int
    dimensions: builtins.int
    def __init__(
        self,
        *,
        dimensions: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dimensions", b"dimensions"]) -> None: ...

global___VectorCommandSchema = VectorCommandSchema

@typing.final
class VectorCommandValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.float]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[builtins.float] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___VectorCommandValue = VectorCommandValue

@typing.final
class StateTensorSchema(google.protobuf.message.Message):
    """------------ *
    State Tensor *
    ------------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SHAPE_FIELD_NUMBER: builtins.int
    DTYPE_FIELD_NUMBER: builtins.int
    dtype: global___DType.ValueType
    @property
    def shape(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    def __init__(
        self,
        *,
        shape: collections.abc.Iterable[builtins.int] | None = ...,
        dtype: global___DType.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "shape", b"shape"]) -> None: ...

global___StateTensorSchema = StateTensorSchema

@typing.final
class StateTensorValue(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___StateTensorValue = StateTensorValue

@typing.final
class Value(google.protobuf.message.Message):
    """----- *
    Value *
    -----
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_NAME_FIELD_NUMBER: builtins.int
    JOINT_POSITIONS_FIELD_NUMBER: builtins.int
    JOINT_VELOCITIES_FIELD_NUMBER: builtins.int
    JOINT_TORQUES_FIELD_NUMBER: builtins.int
    JOINT_COMMANDS_FIELD_NUMBER: builtins.int
    CAMERA_FRAME_FIELD_NUMBER: builtins.int
    AUDIO_FRAME_FIELD_NUMBER: builtins.int
    IMU_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    VECTOR_COMMAND_FIELD_NUMBER: builtins.int
    STATE_TENSOR_FIELD_NUMBER: builtins.int
    value_name: builtins.str
    @property
    def joint_positions(self) -> global___JointPositionsValue: ...
    @property
    def joint_velocities(self) -> global___JointVelocitiesValue: ...
    @property
    def joint_torques(self) -> global___JointTorquesValue: ...
    @property
    def joint_commands(self) -> global___JointCommandsValue: ...
    @property
    def camera_frame(self) -> global___CameraFrameValue: ...
    @property
    def audio_frame(self) -> global___AudioFrameValue: ...
    @property
    def imu(self) -> global___ImuValue: ...
    @property
    def timestamp(self) -> global___TimestampValue: ...
    @property
    def vector_command(self) -> global___VectorCommandValue: ...
    @property
    def state_tensor(self) -> global___StateTensorValue: ...
    def __init__(
        self,
        *,
        value_name: builtins.str = ...,
        joint_positions: global___JointPositionsValue | None = ...,
        joint_velocities: global___JointVelocitiesValue | None = ...,
        joint_torques: global___JointTorquesValue | None = ...,
        joint_commands: global___JointCommandsValue | None = ...,
        camera_frame: global___CameraFrameValue | None = ...,
        audio_frame: global___AudioFrameValue | None = ...,
        imu: global___ImuValue | None = ...,
        timestamp: global___TimestampValue | None = ...,
        vector_command: global___VectorCommandValue | None = ...,
        state_tensor: global___StateTensorValue | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "audio_frame",
            b"audio_frame",
            "camera_frame",
            b"camera_frame",
            "imu",
            b"imu",
            "joint_commands",
            b"joint_commands",
            "joint_positions",
            b"joint_positions",
            "joint_torques",
            b"joint_torques",
            "joint_velocities",
            b"joint_velocities",
            "state_tensor",
            b"state_tensor",
            "timestamp",
            b"timestamp",
            "value",
            b"value",
            "vector_command",
            b"vector_command",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "audio_frame",
            b"audio_frame",
            "camera_frame",
            b"camera_frame",
            "imu",
            b"imu",
            "joint_commands",
            b"joint_commands",
            "joint_positions",
            b"joint_positions",
            "joint_torques",
            b"joint_torques",
            "joint_velocities",
            b"joint_velocities",
            "state_tensor",
            b"state_tensor",
            "timestamp",
            b"timestamp",
            "value",
            b"value",
            "value_name",
            b"value_name",
            "vector_command",
            b"vector_command",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["value", b"value"]
    ) -> (
        typing.Literal[
            "joint_positions",
            "joint_velocities",
            "joint_torques",
            "joint_commands",
            "camera_frame",
            "audio_frame",
            "imu",
            "timestamp",
            "vector_command",
            "state_tensor",
        ]
        | None
    ): ...

global___Value = Value

@typing.final
class ValueSchema(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_NAME_FIELD_NUMBER: builtins.int
    JOINT_POSITIONS_FIELD_NUMBER: builtins.int
    JOINT_VELOCITIES_FIELD_NUMBER: builtins.int
    JOINT_TORQUES_FIELD_NUMBER: builtins.int
    JOINT_COMMANDS_FIELD_NUMBER: builtins.int
    CAMERA_FRAME_FIELD_NUMBER: builtins.int
    AUDIO_FRAME_FIELD_NUMBER: builtins.int
    IMU_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    VECTOR_COMMAND_FIELD_NUMBER: builtins.int
    STATE_TENSOR_FIELD_NUMBER: builtins.int
    value_name: builtins.str
    @property
    def joint_positions(self) -> global___JointPositionsSchema: ...
    @property
    def joint_velocities(self) -> global___JointVelocitiesSchema: ...
    @property
    def joint_torques(self) -> global___JointTorquesSchema: ...
    @property
    def joint_commands(self) -> global___JointCommandsSchema: ...
    @property
    def camera_frame(self) -> global___CameraFrameSchema: ...
    @property
    def audio_frame(self) -> global___AudioFrameSchema: ...
    @property
    def imu(self) -> global___ImuSchema: ...
    @property
    def timestamp(self) -> global___TimestampSchema: ...
    @property
    def vector_command(self) -> global___VectorCommandSchema: ...
    @property
    def state_tensor(self) -> global___StateTensorSchema: ...
    def __init__(
        self,
        *,
        value_name: builtins.str = ...,
        joint_positions: global___JointPositionsSchema | None = ...,
        joint_velocities: global___JointVelocitiesSchema | None = ...,
        joint_torques: global___JointTorquesSchema | None = ...,
        joint_commands: global___JointCommandsSchema | None = ...,
        camera_frame: global___CameraFrameSchema | None = ...,
        audio_frame: global___AudioFrameSchema | None = ...,
        imu: global___ImuSchema | None = ...,
        timestamp: global___TimestampSchema | None = ...,
        vector_command: global___VectorCommandSchema | None = ...,
        state_tensor: global___StateTensorSchema | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "audio_frame",
            b"audio_frame",
            "camera_frame",
            b"camera_frame",
            "imu",
            b"imu",
            "joint_commands",
            b"joint_commands",
            "joint_positions",
            b"joint_positions",
            "joint_torques",
            b"joint_torques",
            "joint_velocities",
            b"joint_velocities",
            "state_tensor",
            b"state_tensor",
            "timestamp",
            b"timestamp",
            "value_type",
            b"value_type",
            "vector_command",
            b"vector_command",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "audio_frame",
            b"audio_frame",
            "camera_frame",
            b"camera_frame",
            "imu",
            b"imu",
            "joint_commands",
            b"joint_commands",
            "joint_positions",
            b"joint_positions",
            "joint_torques",
            b"joint_torques",
            "joint_velocities",
            b"joint_velocities",
            "state_tensor",
            b"state_tensor",
            "timestamp",
            b"timestamp",
            "value_name",
            b"value_name",
            "value_type",
            b"value_type",
            "vector_command",
            b"vector_command",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["value_type", b"value_type"]
    ) -> (
        typing.Literal[
            "joint_positions",
            "joint_velocities",
            "joint_torques",
            "joint_commands",
            "camera_frame",
            "audio_frame",
            "imu",
            "timestamp",
            "vector_command",
            "state_tensor",
        ]
        | None
    ): ...

global___ValueSchema = ValueSchema

@typing.final
class IOSchema(google.protobuf.message.Message):
    """---------------- *
    Inputs / Outputs *
    ----------------
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValueSchema]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___ValueSchema] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___IOSchema = IOSchema

@typing.final
class IO(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUES_FIELD_NUMBER: builtins.int
    @property
    def values(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Value]: ...
    def __init__(
        self,
        *,
        values: collections.abc.Iterable[global___Value] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["values", b"values"]) -> None: ...

global___IO = IO

@typing.final
class ModelSchema(google.protobuf.message.Message):
    """----- *
    Model *
    -----
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUT_SCHEMA_FIELD_NUMBER: builtins.int
    OUTPUT_SCHEMA_FIELD_NUMBER: builtins.int
    @property
    def input_schema(self) -> global___IOSchema: ...
    @property
    def output_schema(self) -> global___IOSchema: ...
    def __init__(
        self,
        *,
        input_schema: global___IOSchema | None = ...,
        output_schema: global___IOSchema | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["input_schema", b"input_schema", "output_schema", b"output_schema"]
    ) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["input_schema", b"input_schema", "output_schema", b"output_schema"]
    ) -> None: ...

global___ModelSchema = ModelSchema
